import cmd2

from GLXShell.libs.properties.shortcuts import GLXShPropertyShortcuts
from GLXShell.libs.properties.history import GLXShPropertyHistory
from GLXShell.libs.properties.config import GLXShPropertyConfig
from GLXShell.libs.properties.shell import GLXShPropertyShell
from GLXShell.libs.utils.get_terminal_size import get_terminal_size
from GLXShell.libs.plugins import GLXShPluginsManager
from GLXShell.libs.intro import GLXShIntro
from GLXShell.libs.prompt import GLXShPrompt
from GLXShell.libs.settable import GLXShSettable


# Key binding
# http://readline.kablamo.org/emacs.html


class GLXShell(
    GLXShPropertyShortcuts,
    GLXShPropertyHistory,
    GLXShPropertyConfig,
    GLXShPropertyShell,
    GLXShIntro,
    GLXShPrompt,
    GLXShPluginsManager,
    cmd2.Cmd,
    GLXShSettable,
):
    def __init__(self):
        GLXShPropertyShortcuts.__init__(self)
        GLXShPropertyHistory.__init__(self)
        GLXShPropertyConfig.__init__(self)
        GLXShPropertyShell.__init__(self)
        GLXShIntro.__init__(self)
        GLXShPrompt.__init__(self)

        # Init and config the cmd2 object
        cmd2.Cmd.__init__(
            self,
            persistent_history_file=self.persistent_history_file,
            persistent_history_length=self.persistent_history_length,
            shortcuts=self.shortcuts,
            auto_load_commands=False,
            allow_cli_args=True,
        )
        self.allow_redirection = True
        self.default_to_shell = True

        # PostProcessing
        GLXShPluginsManager(self).load_plugins()
        GLXShSettable(self).load_settable()

    @property
    def intro(self):
        """Return intro to displays"""
        self.columns, self.rows = get_terminal_size()
        return self.intro_to_display

    @property
    def prompt(self):
        """Return prompt to displays"""
        return self.prompt_to_display



import os
import cmd2
import argparse

from GLXShell.plugins.builtins import __version__
from GLXShell.plugins.builtins import __appname__
from GLXShell.plugins.builtins import __licence__

which_parser = cmd2.Cmd2ArgumentParser(
    description="which  returns  the pathnames of the files (or links) which would be executed in the current "
                "environment, had its arguments been given as commands in a strictly POSIX-conformant shell.  It does "
                "this by searching the PATH for executable files matching the names  of  the arguments. It does not "
                "canonicalize path names. "
)
which_parser.add_argument(
    "-a",
    "--all",
    action="store_true",
    help="print all matching pathnames of each argument",
)

which_parser.add_argument(
    "--version", action="store_true", help="output version information and exit"
)

which_parser.add_argument("filename", nargs=argparse.ZERO_OR_MORE)


class GLXWhich(cmd2.CommandSet):
    def __init__(self):
        super().__init__()

    @staticmethod
    def which_print_version():
        cmd2.Cmd().poutput(
            "which ({name}) v{version}\n{licence}".format(
                name=__appname__,
                version=__version__,
                licence=__licence__,
            )
        )

    @staticmethod
    def which(all_info=False, filename=None):
        for exec_path in os.get_exec_path():
            if os.path.isfile(os.path.join(exec_path, filename)):
                if os.access(os.path.join(exec_path, filename), os.X_OK):
                    cmd2.Cmd().poutput(os.path.join(exec_path, filename))
                    if not all_info:
                        return

    @cmd2.with_argparser(which_parser)
    @cmd2.with_category("Builtins")
    def do_which(self, args):
        if args.version:  # pragma: no cover
            self.which_print_version()
            return

        if not args.filename or len(args.filename) > 1:  # pragma: no cover
            which_parser.print_help()
            return

        self.which(all_info=args.all, filename=args.filename[0])  # pragma: no cover



__authors__ = ["Tuxa"]
__date__ = "20201104"
__version__ = "0.1a2"
__appname__ = "glxsh builtins plugin"
__description__ = "Galaxie Shell"
__licence__ = """Copyright (C) 2020 Galaxie Shell Project.
License GPLv3+: GNU GPL version 3 or later <https://gnu.org/licenses/gpl.html>.
This is free software: you are free to change and redistribute it.
There is NO WARRANTY, to the extent permitted by law.
"""

from GLXShell.libs.plugin import GLXShellPlugin
from GLXShell.plugins.builtins.arch import GLXArch
from GLXShell.plugins.builtins.cat import GLXCat
from GLXShell.plugins.builtins.cd import GLXCd
from GLXShell.plugins.builtins.dir import GLXDir
from GLXShell.plugins.builtins.mkdir import GLXMkdir
from GLXShell.plugins.builtins.pwd import GLXPwd
from GLXShell.plugins.builtins.rmdir import GLXRmDir
from GLXShell.plugins.builtins.uname import GLXUname
from GLXShell.plugins.builtins.which import GLXWhich


class GLXShPluginBuiltins(GLXShellPlugin):
    def __init__(self):
        GLXShellPlugin.__init__(self)
        self.commands = [
            {"name": "arch", "object": GLXArch()},
            {"name": "cat", "object": GLXCat()},
            {"name": "cd", "object": GLXCd()},
            {"name": "dir", "object": GLXDir()},
            {"name": "mkdir", "object": GLXMkdir()},
            {"name": "pwd", "object": GLXPwd()},
            {"name": "rmdir", "object": GLXRmDir()},
            {"name": "uname", "object": GLXUname()},
            {"name": "which", "object": GLXWhich()},
        ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# It script it publish under GNU GENERAL PUBLIC LICENSE
# http://www.gnu.org/licenses/gpl-3.0.en.html
# Author: the Galaxie Shell Team, all rights reserved

import argparse
import cmd2

from GLXShell.plugins.builtins import GLXShPluginBuiltins
from GLXShell.libs.config import GLXShConfig

managed = [
    {"name": "builtins", "object": GLXShPluginBuiltins()},
]


def choices():
    to_return = []
    for plugin in managed:
        if plugin and "name" in plugin:
            to_return.append(plugin["name"])
    return to_return


class GLXShPluginsManager(object):
    def __init__(self, *args, **kwargs):
        self.config = kwargs.get("config", GLXShConfig())
        self.shell = kwargs.get("shell", args[0])

        have_change = False

        if "plugins" not in self.config.data:
            self.config.data["plugins"] = {}
            have_change = True

        for plugin in managed:
            if plugin["name"] not in self.config.data["plugins"]:
                self.config.data["plugins"][plugin["name"]] = {}
                have_change = True
            if "enabled" not in self.config.data["plugins"][plugin["name"]]:
                self.config.data["plugins"][plugin["name"]]["enabled"] = True
                have_change = True

        if have_change:
            self.config.write()

    def load_plugins(self):
        for plugin in managed:
            if self.config.data["plugins"][plugin["name"]]["enabled"]:
                plugin["object"].shell = self.shell
                plugin["object"].load()

    enable_parser = cmd2.Cmd2ArgumentParser()
    enable_parser.add_argument("name", choices=choices())

    @cmd2.as_subcommand_to("plugins", "enable", enable_parser)
    @cmd2.with_category("Plugins commands")
    def enable(self, ns: argparse.Namespace):
        """Enable at startup a plugin by it name and load it"""
        for plugin in managed:
            if ns.name == plugin["name"]:
                plugin["object"].shell = self.shell
                plugin["object"].load()
                if not self.config.data["plugins"][plugin["name"]]["enabled"]:
                    self.config.data["plugins"][plugin["name"]]["enabled"] = True
                    self.config.write()

    disable_parser = cmd2.Cmd2ArgumentParser()
    disable_parser.add_argument("name", choices=choices())

    @cmd2.as_subcommand_to("plugins", "disable", disable_parser)
    @cmd2.with_category("Plugins commands")
    def disable(self, ns: argparse.Namespace):
        """Disable at startup a plugin by it name and unload it"""
        for plugin in managed:
            if ns.name == plugin["name"]:
                plugin["object"].shell = self.shell
                plugin["object"].unload()
                if self.config.data["plugins"][plugin["name"]]["enabled"]:
                    self.config.data["plugins"][plugin["name"]]["enabled"] = False
                    self.config.write()

    load_parser = cmd2.Cmd2ArgumentParser()
    load_parser.add_argument("name", choices=choices())

    @cmd2.as_subcommand_to("plugins", "load", load_parser)
    @cmd2.with_category("Plugins commands")
    def load(self, ns: argparse.Namespace):
        """Load a plugin by it name"""
        for plugin in managed:
            if ns.name == plugin["name"]:
                plugin["object"].shell = self.shell
                plugin["object"].load()

    unload_parser = cmd2.Cmd2ArgumentParser()
    unload_parser.add_argument("name", choices=choices())

    @cmd2.as_subcommand_to("plugins", "unload", unload_parser)
    @cmd2.with_category("Plugins commands")
    def unload(self, ns: argparse.Namespace):
        """Unload a plugin by it name"""
        for plugin in managed:
            if ns.name == plugin["name"]:
                plugin["object"].shell = self.shell
                plugin["object"].unload()

    reload_parser = cmd2.Cmd2ArgumentParser()
    reload_parser.add_argument("name", choices=choices())

    @cmd2.as_subcommand_to("plugins", "reload", reload_parser)
    @cmd2.with_category("Plugins commands")
    def reload(self, ns: argparse.Namespace):
        """Reload a plugin by it name"""
        for plugin in managed:
            if ns.name == plugin["name"]:
                plugin["object"].shell = self.shell
                plugin["object"].reload()

    plugins_parser = cmd2.Cmd2ArgumentParser(description="Plugins management")
    plugins_subparsers = plugins_parser.add_subparsers(
        title="commands", help="a command it can be choose"
    )

    @cmd2.with_argparser(plugins_parser)
    @cmd2.with_category("Plugins")
    def do_plugins(self, ns: argparse.Namespace):
        handler = ns.cmd2_handler.get()
        if handler is not None:
            # Call whatever subcommand function was selected
            handler(ns)
        else:
            # No subcommand was provided, so call help
            cmd2.Cmd().perror("This command does nothing without sub-commands")
            self.shell.do_help("plugins")



__authors__ = ["Tuxa"]
__date__ = "20201102"
__version__ = "0.1a4"
__appname__ = "glxsh"
__description__ = "Galaxie Shell"
__licence__ = """Copyright (C) 2020 Galaxie Shell Project.
License GPLv3+: GNU GPL version 3 or later <https://gnu.org/licenses/gpl.html>.
This is free software: you are free to change and redistribute it.
There is NO WARRANTY, to the extent permitted by law.
"""



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# It script it publish under GNU GENERAL PUBLIC LICENSE
# http://www.gnu.org/licenses/gpl-3.0.en.html
# Author: the Galaxie Curses Team, all rights reserved

import os
import getpass
import socket

from GLXShell.libs.config import GLXShConfig


class GLXShPrompt(object):
    def __init__(self, *args, **kwargs):
        self.config = kwargs.get("config", GLXShConfig())

        have_change = False

        if "prompt" not in self.config.data:
            self.config.data["prompt"] = {}
            have_change = True

        if "show" not in self.config.data["prompt"]:
            self.config.data["prompt"]["show"] = {}
            have_change = True

        if have_change:
            self.config.write()

        self.__prompt_show_info = None
        self.__prompt_show_cursor = None

        try:
            self.prompt_show_cursor = self.config.data["prompt"]["show"]["cursor"]
        except KeyError:
            self.prompt_show_cursor = None
        try:
            self.prompt_show_info = self.config.data["prompt"]["show"]["info"]
        except KeyError:
            self.prompt_show_info = None

    @property
    def prompt_show_info(self):
        return self.__prompt_show_info

    @prompt_show_info.setter
    def prompt_show_info(self, value=None):
        if value is None:
            value = False
        if type(value) != bool:
            raise TypeError(
                "'prompt_show_info' property value must be a bool type or None"
            )
        if self.prompt_show_info != value:
            self.__prompt_show_info = value
        if (
            "info" not in self.config.data["prompt"]["show"]
            or self.config.data["prompt"]["show"]["info"] != value
        ):
            self.config.data["prompt"]["show"]["info"] = value
            self.config.write()

    @property
    def prompt_show_cursor(self):
        return self.__prompt_show_cursor

    @prompt_show_cursor.setter
    def prompt_show_cursor(self, value=None):
        if value is None:
            value = True
        if type(value) != bool:
            raise TypeError(
                "'prompt_show_cursor' property value must be a bool type or None"
            )
        if self.prompt_show_cursor != value:
            self.__prompt_show_cursor = value
        if (
            "cursor" not in self.config.data["prompt"]["show"]
            or self.config.data["prompt"]["show"]["cursor"] != value
        ):
            self.config.data["prompt"]["show"]["cursor"] = value
            self.config.write()

    def onchange_prompt_show_cursor(self, param_name, old_value, new_value):
        pass

    def onchange_prompt_show_info(self, param_name, old_value, new_value):
        pass

    @property
    def prompt_env_is_virtual(self):
        if os.environ.get("VIRTUAL_ENV") or os.environ.get("CONDA_DEFAULT_ENV"):
            return True
        else:
            return False

    @property
    def prompt_env_text(self):
        if os.environ.get("VIRTUAL_ENV") or os.environ.get("CONDA_DEFAULT_ENV"):
            env_path = os.environ.get("VIRTUAL_ENV", "")
            if len(env_path) == 0:
                env_path = os.environ.get("CONDA_DEFAULT_ENV", "")

            return "({env_name}) ".format(env_name=os.path.basename(env_path))
        else:
            return ""

    @property
    def prompt_username_text(self):
        return getpass.getuser()

    @property
    def prompt_hostname_text(self):
        return socket.gethostname()

    @property
    def prompt_path_text(self):
        return os.getcwd().replace(os.path.realpath(os.path.expanduser("~")), "~")

    @property
    def prompt_symbol_text(self):
        if os.getuid() == 0:  # pragma: no cover
            return "#"
        else:
            return "$"

    @property
    def prompt_cursor_text(self):
        return ">"

    @property
    def prompt_to_display(self):
        """Set prompt so it displays the current working directory."""
        prompt = []
        if self.prompt_show_info:
            prompt.append(
                "{venv}{username}@{hostname}:{path} {symbol} ".format(
                    venv=self.prompt_env_text,
                    username=self.prompt_username_text,
                    hostname=self.prompt_hostname_text,
                    path=self.prompt_path_text,
                    symbol=self.prompt_symbol_text,
                )
            )
        if self.prompt_show_cursor:
            prompt.append("{cursor} ".format(cursor=self.prompt_cursor_text))

        return "\n".join(prompt)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# It script it publish under GNU GENERAL PUBLIC LICENSE
# http://www.gnu.org/licenses/gpl-3.0.en.html
# Author: the Galaxie Shell Team, all rights reserved

import os


def get_memory_total():
    try:
        return os.sysconf("SC_PAGE_SIZE") * os.sysconf("SC_PHYS_PAGES")
    except ValueError:  # pragma: no cover
        return 0


def get_memory_available():
    try:
        return os.sysconf("SC_PAGE_SIZE") * os.sysconf("SC_AVPHYS_PAGES")
    except ValueError:  # pragma: no cover
        return 0


def get_size(size, suffix="B"):
    """
    Scale bytes to its proper format
    e.g:
        1253656 => '1.20MB'
        1253656678 => '1.17GB'
    """
    factor = 1024
    for unit in ["", "K", "M", "G", "T", "P"]:
        if size < factor:
            return f"{size:.2f}{unit}{suffix}"
        size /= factor



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# It script it publish under GNU GENERAL PUBLIC LICENSE
# http://www.gnu.org/licenses/gpl-3.0.en.html
# Author: the Galaxie Shell Team, all rights reserved

import os
import json

from GLXShell.libs.utils.xdg_base_directory import XDGBaseDirectory
from GLXShell.libs.utils.singleton import Singleton


class GLXShConfig(XDGBaseDirectory, metaclass=Singleton):
    def __init__(self):
        XDGBaseDirectory.__init__(self)

        self.__file = None
        self.__data = None

        self.file = None
        self.data = None

        self.touch()
        self.load()

    @property
    def file(self):
        """
        Property it store configuration file path

        Note: Default is XDGBaseDirectory/glxsh

        :return: the config file path
        :rtype: str
        """
        return self.__file

    @file.setter
    def file(self, value=None):
        """
        Set the ``config_file_path`` property value

        Note: if ``config_file_path`` property value is set to ``None``
        it will restore default value: XDGBaseDirectory/glxsh

        :param value: the config file path
        :type value: str or None
        """
        if value is None:
            value = os.path.join(self.config_path, "config.json")
        if type(value) != str:
            raise TypeError("'config_file' property value must be a str type or None")
        if value != self.file:
            self.__file = value
            # self.load()

    @property
    def data(self):
        """
        A dictionary it contain all the setting

        :return: the setting
        :rtype: dict
        """
        return self.__data

    @data.setter
    def data(self, value=None):
        """
        Set the ``data`` property

        Note: If ``data`` property value is ``None`` is restore a empty dictionary setting

        :param value: the setting
        :return: dict or None
        :raise TypeError: When ``data`` property value is not dict type or None
        """
        if value is None:
            value = {}
        if type(value) != dict:
            raise TypeError("'data' property value must be a dict type or None")
        if self.data != value:
            self.__data = value

    def touch(self):
        """Create directory where store the configuration file and if the config file do not exist
        create one with a empty dictionary
        """
        if not os.path.exists(os.path.dirname(self.file)):
            os.makedirs(os.path.dirname(self.file))
        if not os.path.exists(self.file):
            with open(self.file, "w") as outfile:
                json.dump({}, outfile)

    def load(self):
        """Load the configuration file and update ``data`` property"""
        with open(self.file, "r") as json_data_file:
            self.data.update(json.load(json_data_file))

    def write(self):
        """Write ``data`` property content inside the configuration file"""
        with open(self.file, "w") as outfile:
            json.dump(self.data, outfile, sort_keys=True, indent=4)



import os
import io

from GLXShell.libs.utils.id import new_id


def get_os_temporary_dir():
    """
    Get the OS default dir , the better as it can.

    It suppose to be cross platform

    :return: A tmp dir path
    :rtype: str
    """
    text_flags = os.O_RDWR | os.O_CREAT | os.O_EXCL
    if hasattr(os, "O_NOFOLLOW"):
        text_flags |= os.O_NOFOLLOW

    bin_flags = text_flags
    if hasattr(os, "O_BINARY"):  # pragma: no cover
        bin_flags |= os.O_BINARY

    directory_list = list()

    # First, try the environment.
    for envname in "TMPDIR", "TEMP", "TMP":
        dirname = os.getenv(envname)
        if dirname:
            directory_list.append(os.path.abspath(dirname))

    directory_list.extend(["/tmp", "/var/tmp", "/usr/tmp"])

    for directory in directory_list:
        if directory != os.path.curdir:
            directory = os.path.abspath(directory)

        name = str("glxshell-")
        name += str(new_id())
        name += str(".cp")
        filename = os.path.join(directory, name)
        try:
            fd = os.open(filename, bin_flags, 0o600)
            try:
                try:
                    with io.open(fd, "wb", closefd=False) as fp:
                        fp.write(b"Test")
                finally:
                    os.close(fd)
            finally:
                os.unlink(filename)
            return directory

        except PermissionError:  # pragma: no cover
            break  # no point trying more names in this directory
        except OSError:  # pragma: no cover
            break  # no point trying more names in this directory
    raise FileNotFoundError(  # pragma: no cover
        "No usable temporary directory found in %s" % directory_list
    )



import os


def get_terminal_size():
    fallback = (80, 24)
    try:
        from shutil import get_terminal_size
    except ImportError:  # pragma: no cover
        try:
            import termios
            import fcntl
            import struct

            call = fcntl.ioctl(0, termios.TIOCGWINSZ, "\x00" * 8)
            height, width = struct.unpack("hhhh", call)[:2]
        except (SystemExit, KeyboardInterrupt):
            raise
        except:
            width = int(os.environ.get("COLUMNS", fallback[0]))
            height = int(os.environ.get("COLUMNS", fallback[1]))
        # Work around above returning width, height = 0, 0 in Emacs
        width = width if width != 0 else fallback[0]
        height = height if height != 0 else fallback[1]
        return width, height
    else:
        return get_terminal_size(fallback)



# !/usr/bin/env python
# -*- coding: utf-8 -*-

# It script it publish under GNU GENERAL PUBLIC LICENSE
# http://www.gnu.org/licenses/gpl-3.0.en.html
# Author: the Galaxie Shell Team, all rights reserved

import os
import sys
import platform

from GLXShell import __version__
from GLXShell import __appname__
from GLXShell.libs.utils.info_memory import get_memory_total
from GLXShell.libs.utils.info_memory import get_memory_available
from GLXShell.libs.utils.info_memory import get_size
from GLXShell.libs.config import GLXShConfig


class GLXShIntro(object):
    def __init__(self, *args, **kwargs):
        self.config = kwargs.get("config", GLXShConfig())

        have_change = False
        if "intro" not in self.config.data:
            self.config.data["intro"] = {}
            have_change = True

        if "show" not in self.config.data["intro"]:
            self.config.data["intro"]["show"] = {}
            have_change = True

        if have_change:
            self.config.write()

        self.rows = 24
        self.columns = 80

        self.__intro_show_title = None
        self.__intro_show_spacing = None
        self.__intro_show_license = None
        self.__intro_show_loader = None
        self.__intro_show_exec = None
        self.__intro_show_memory_total = None
        self.__intro_show_memory_free = None
        self.__intro_show_holotape = None
        self.__intro_show_rom = None

        try:
            self.intro_show_exec = self.config.data["intro"]["show"]["exec"]
        except KeyError:
            self.intro_show_exec = None
        try:
            self.intro_show_holotape = self.config.data["intro"]["show"]["holotape"]
        except KeyError:
            self.intro_show_holotape = None
        try:
            self.intro_show_license = self.config.data["intro"]["show"]["license"]
        except KeyError:
            self.intro_show_license = None
        try:
            self.intro_show_loader = self.config.data["intro"]["show"]["loader"]
        except KeyError:
            self.intro_show_loader = None
        try:
            self.intro_show_memory_free = self.config.data["intro"]["show"][
                "memory_free"
            ]
        except KeyError:
            self.intro_show_memory_free = None
        try:
            self.intro_show_memory_total = self.config.data["intro"]["show"][
                "memory_total"
            ]
        except KeyError:
            self.intro_show_memory_total = None
        try:
            self.intro_show_rom = self.config.data["intro"]["show"]["rom"]
        except KeyError:
            self.intro_show_rom = None
        try:
            self.intro_show_spacing = self.config.data["intro"]["show"]["spacing"]
        except KeyError:
            self.intro_show_spacing = None
        try:
            self.intro_show_title = self.config.data["intro"]["show"]["title"]
        except KeyError:
            self.intro_show_title = None

    @property
    def title(self):
        return "{app_name} V{version}".format(
            app_name=__appname__.upper(), version=__version__.upper()
        )

    @property
    def title_line(self):
        return "{text_inner} {text} {text_outer}".format(
            text=self.title,
            text_inner="*" * int((int(self.columns) / 2) - (len(self.title) / 2) - 1),
            text_outer="*" * int((int(self.columns) / 2) - (len(self.title) / 2) - 1),
        )

    @property
    def intro_show_exec(self):
        return self.__intro_show_exec

    @intro_show_exec.setter
    def intro_show_exec(self, value=None):
        if value is None:
            value = True
        if type(value) != bool:
            raise TypeError("'exec' property value must be a bool type or None")
        if self.intro_show_exec != value:
            self.__intro_show_exec = value
        if (
            "exec" not in self.config.data["intro"]["show"]
            or self.config.data["intro"]["show"]["exec"] != value
        ):
            self.config.data["intro"]["show"]["exec"] = value
            self.config.write()

    @property
    def intro_show_holotape(self):
        return self.__intro_show_holotape

    @intro_show_holotape.setter
    def intro_show_holotape(self, value=None):
        if value is None:
            value = True
        if type(value) != bool:
            raise TypeError("'holotape' property value must be a bool type or None")
        if self.intro_show_holotape != value:
            self.__intro_show_holotape = value
        if (
            "holotape" not in self.config.data["intro"]["show"]
            or self.config.data["intro"]["show"]["holotape"] != value
        ):
            self.config.data["intro"]["show"]["holotape"] = value
            self.config.write()

    @property
    def intro_show_license(self):
        return self.__intro_show_license

    @intro_show_license.setter
    def intro_show_license(self, value=None):
        if value is None:
            value = True
        if type(value) != bool:
            raise TypeError("'license' property value must be a bool type or None")
        if self.intro_show_license != value:
            self.__intro_show_license = value
        if (
            "license" not in self.config.data["intro"]["show"]
            or self.config.data["intro"]["show"]["license"] != value
        ):
            self.config.data["intro"]["show"]["license"] = value
            self.config.write()

    @property
    def intro_show_loader(self):
        return self.__intro_show_loader

    @intro_show_loader.setter
    def intro_show_loader(self, value=None):
        if value is None:
            value = True
        if type(value) != bool:
            raise TypeError("'loader' property value must be a bool type or None")
        if self.intro_show_loader != value:
            self.__intro_show_loader = value
        if (
            "loader" not in self.config.data["intro"]["show"]
            or self.config.data["intro"]["show"]["loader"] != value
        ):
            self.config.data["intro"]["show"]["loader"] = value
            self.config.write()

    @property
    def intro_show_memory_free(self):
        return self.__intro_show_memory_free

    @intro_show_memory_free.setter
    def intro_show_memory_free(self, value=None):
        if value is None:
            value = True
        if type(value) != bool:
            raise TypeError("'memory_free' property value must be a bool type or None")
        if self.intro_show_memory_free != value:
            self.__intro_show_memory_free = value
        if (
            "memory_free" not in self.config.data["intro"]["show"]
            or self.config.data["intro"]["show"]["memory_free"] != value
        ):
            self.config.data["intro"]["show"]["memory_free"] = value
            self.config.write()

    @property
    def intro_show_memory_total(self):
        return self.__intro_show_memory_total

    @intro_show_memory_total.setter
    def intro_show_memory_total(self, value=None):
        if value is None:
            value = True
        if type(value) != bool:
            raise TypeError("'memory_total' property value must be a bool type or None")
        if self.intro_show_memory_total != value:
            self.__intro_show_memory_total = value
        if (
            "memory_total" not in self.config.data["intro"]["show"]
            or self.config.data["intro"]["show"]["memory_total"] != value
        ):
            self.config.data["intro"]["show"]["memory_total"] = value
            self.config.write()

    @property
    def intro_show_rom(self):
        return self.__intro_show_rom

    @intro_show_rom.setter
    def intro_show_rom(self, value=None):
        if value is None:
            value = True
        if type(value) != bool:
            raise TypeError("'rom' property value must be a bool type or None")
        if self.intro_show_rom != value:
            self.__intro_show_rom = value
        if (
            "rom" not in self.config.data["intro"]["show"]
            or self.config.data["intro"]["show"]["rom"] != value
        ):
            self.config.data["intro"]["show"]["rom"] = value
            self.config.write()

    @property
    def intro_show_spacing(self):
        return self.__intro_show_spacing

    @intro_show_spacing.setter
    def intro_show_spacing(self, value=None):
        if value is None:
            value = True
        if type(value) != bool:
            raise TypeError(
                "'intro_show_spacing' property value must be a bool type or None"
            )
        if self.intro_show_spacing != value:
            self.__intro_show_spacing = value
        if (
            "spacing" not in self.config.data["intro"]["show"]
            or self.config.data["intro"]["show"]["spacing"] != value
        ):
            self.config.data["intro"]["show"]["spacing"] = value
            self.config.write()

    @property
    def intro_show_title(self):
        return self.__intro_show_title

    @intro_show_title.setter
    def intro_show_title(self, value=None):
        if value is None:
            value = True
        if type(value) != bool:
            raise TypeError(
                "'intro_show_title' property value must be a bool type or None"
            )
        if self.intro_show_title != value:
            self.__intro_show_title = value
        if (
            "title" not in self.config.data["intro"]["show"]
            or self.config.data["intro"]["show"]["title"] != value
        ):
            self.config.data["intro"]["show"]["title"] = value
            self.config.write()

    def onchange_intro_show_exec(self, param_name, old_value, new_value):
        pass

    def onchange_intro_show_holotape(self, param_name, old_value, new_value):
        pass

    def onchange_intro_show_license(self, param_name, old_value, new_value):
        pass

    def onchange_intro_show_loader(self, param_name, old_value, new_value):
        pass

    def onchange_intro_show_memory_free(self, param_name, old_value, new_value):
        pass

    def onchange_intro_show_memory_total(self, param_name, old_value, new_value):
        pass

    def onchange_intro_show_rom(self, param_name, old_value, new_value):
        pass

    def onchange_intro_show_spacing(self, param_name, old_value, new_value):
        pass

    def onchange_intro_show_title(self, param_name, old_value, new_value):
        pass

    @property
    def intro_to_display(self):
        if os.environ.get("VIRTUAL_ENV") or os.environ.get("CONDA_DEFAULT_ENV"):
            exec_venv = "VENV"
        else:
            exec_venv = "ENV"
        to_return = []
        if self.intro_show_title:
            to_return.append(self.title_line)
            if self.intro_show_spacing:
                to_return.append("")
                to_return.append("")
        if self.intro_show_license:
            to_return.append("GNU GENERAL PUBLIC LICENSE GPL-3.0")
        if self.intro_show_loader:
            to_return.append("LOADER {0}".format(platform.version().upper()))
        if self.intro_show_exec:
            to_return.append(
                "EXEC {exec_venv} PYTHON {python_version_major}.{python_version_minor}.{python_version_micro}".format(
                    exec_venv=exec_venv,
                    python_version_major=sys.version_info.major,
                    python_version_minor=sys.version_info.minor,
                    python_version_micro=sys.version_info.micro,
                )
            )
        if self.intro_show_memory_total:
            to_return.append(
                "{mem_total} RAM SYSTEM".format(mem_total=get_size(get_memory_total()))
            )
        if self.intro_show_memory_free:
            to_return.append(
                "{mem_available} FREE".format(
                    mem_available=get_size(get_memory_available())
                )
            )
        if self.intro_show_holotape:
            to_return.append("NO HOLOTAPE FOUND")
        if self.intro_show_rom:
            to_return.append("LOAD ROM(1): DEITRIX 303")
        if self.intro_show_spacing:
            to_return.append("")

        return "\n".join(to_return)



import cmd2.plugin
from GLXShell.libs.properties.shell import GLXShPropertyShell
from GLXShell.libs.properties.commands import GLXShellPropertyCommands


class GLXShellPlugin(GLXShPropertyShell, GLXShellPropertyCommands):
    def __init__(self):
        GLXShPropertyShell.__init__(self)
        GLXShellPropertyCommands.__init__(self)

    def load(self):
        if self.shell.debug:
            cmd2.Cmd().pwarning("PLUGINS LOAD ...")
        for cmd in self.commands:
            try:
                self.shell.register_command_set(cmd["object"])
                if self.shell.debug:
                    cmd2.Cmd().pwarning("{0}".format(cmd["name"]))
            except cmd2.CommandSetRegistrationError:
                cmd2.Cmd().perror("{0} already loaded".format(cmd["name"]))

    def unload(self):
        if self.shell.debug:
            cmd2.Cmd().pwarning("PLUGINS UNLOAD ...")
        for cmd in self.commands:
            self.shell.unregister_command_set(cmd["object"])
            if self.shell.debug:
                cmd2.Cmd().pwarning("{0}".format(cmd["name"]))

    def reload(self):
        self.unload()
        self.load()
